class SantimPayAPIResponse:

    def __init__(self, error, msg, data):
    
        self.error = error
        self.msg = msg
        self.data = data
    
    @staticmethod
    def fromJson(json):
    
        return SantimPayAPIResponse(json["error"], json["msg"], json["data"])
    



class SantimPayCheckoutRequest:

    def __init__(
        self, 
        id,
        amount,
        paymentReason,
        successRedirectUrl,
        failureRedirectUrl,
        notifyUrl,
        cancelRedirectUrl,
        direct = False
    ):
        self.id = id
        self.amount = amount
        self.paymentReason = paymentReason
        self.successRedirectUrl = successRedirectUrl
        self.failureRedirectUrl = failureRedirectUrl
        self.notifyUrl = notifyUrl
        self.cancelRedirectUrl = cancelRedirectUrl
        self.direct = direct
    

    def jsonSerialize(self):
    
        if self.direct:
            return {
                'id' : self.id,
                'amount' : self.amount,
                'reason' : self.paymentReason,
                'notifyUrl' : self.notifyUrl,
            }
        return {
            'id' : self.id,
            'amount' : self.amount,
            'reason' : self.paymentReason,
            'successRedirectUrl' : self.successRedirectUrl,
            'failureRedirectUrl' : self.failureRedirectUrl,
            'notifyUrl' : self.notifyUrl,
            'cancelRedirectUrl' : self.cancelRedirectUrl,
        }
    




from ...types import SantimPayOptions
from ...config import Config
from ...exceptions.santimpay_exceptions import *
from ...types import *
import jwt as JWT
from time import time

class SantimPayDirect:

    def __init__(self, http_client, merchant_id, private_key, payment_method):
    
        self.http_client = http_client
        self.merchant_id = merchant_id
        self.private_key = private_key
        self.payment_method = payment_method
    
    # SantimPayCheckoutRequest SantimPayOptions . SantimPayCheckoutResponse
    def create(self, santimPayCheckoutRequest: SantimPayCheckoutRequest, phone: str,  option: SantimPayOptions = None): 
    
        if option == None:
            option = SantimPayOptions(False)
        

        try:
            santimPayCheckoutRequest.direct = True
            body = santimPayCheckoutRequest.jsonSerialize() 
            body['phoneNumber'] = phone
            body['paymentMethod'] = self.payment_method
            body['merchantId'] = self.merchant_id
            body['signedToken'] = self.generateSignedToken(santimPayCheckoutRequest.amount, santimPayCheckoutRequest.paymentReason, self.payment_method, phone)
            
            response = self.http_client.post(Config.API_VERSION + "/direct-payment", json=body)
            url = response.text.replace('\u0026', '&')

        
            return SantimPayCheckoutResponse.fromJson(url)
        
        except ConnectionErrorException as e: # (ConnectionErrorException e) 
            raise SantimPayNetworkException()
        except Exception as e:# (ClientException e) 
            print("exception")
    

    def generateSignedToken(self, amount, paymentReason, paymentMethod, phone):
    
        time_ = time()
        data = {

            'amount' : amount,
            'paymentMethod' : paymentMethod,
            'phoneNumber' : phone,
            'paymentReason' : paymentReason,
            'merchantId' : self.merchant_id,
            'generated' : time_
        }
        

        jwt = JWT.encode(data, self.private_key, algorithm='ES256')

        return jwt
    






class SantimPayCheckoutResponse:

    def __init__(
        self,
        payment_url
    ):
       
        self.payment_url = payment_url
    

    def jsonSerialize(self):
    
        return {
        
            "payment_url" : self.payment_url,
            
        }
    
    @staticmethod
    def fromJson(url):
    
        return SantimPayCheckoutResponse(url)





class SantimPayCheckoutSession:


    def __init__(self, id, transcation, totalAmount, test, uuid, created_at, update_at):
    
        self.id = id
        self.transaction = transcation
        self.totalAmount = totalAmount
        self.test = test
        self.uuid = uuid
        self.created_at = created_at
        self.update_at = update_at
    

    def jsonSerialize(self):
    
        return {
            "id" : self.id,
            "transaction" : self.transaction,
            "totalAmount" : self.totalAmount,
            "test" : self.test,
            "uuid" : self.uuid,
            "created_at" : self.created_at,
            "update_at" : self.update_at,
        }
    
    @staticmethod
    def fromJson(json: dict):
        
        return SantimPayCheckoutSession(json["id"], json.get("transaction") if json.get("transaction") else None , json["totalAmount"], json["test"], json["uuid"], json["createdAt"], json["updatedAt"])
    




from .api_checkout_request import SantimPayCheckoutRequest
from .api_checkout_response import SantimPayCheckoutResponse
from .api_checkout_session import SantimPayCheckoutSession
from .api_options import SantimPayOptions
from .api_response import SantimPayAPIResponse
from .api_transaction import SantimPayTransaction
from .api_transfer_response import SantimPayTransferResponse


"""
API SDK for SantimPay Payment Gateway
"""
from .config import Config

import httpx
from .core import SantimPayCheckout, SantimPayDirectPay

class SantimPay:
    http_client = ""
    merchant_id = ""
    private_key = ""
    DEFAULT_TIMEOUT = 1000 * 60 * 2
    checkout = ""
    directPay = ""

    def __init__(self, merchant_id: str, private_key: str, test = False):
        """__Santimay Payment gateway__

        Args:
            merchant_id (str): your merchat id from santimpay
            private_key (str): generated by your own
            test (bool, optional): weather if it is in test mode or not. Defaults to False.
        """
        self.merchant_id = merchant_id
        self.private_key = private_key
        base_url = Config.DEFAULT_TEST_HOST if test else Config.DEFAULT_HOST
        headers = {
            "Content-Type": "application/json; charset=UTF-8",
            "Accepts": "application/json",
        }

        self.http_client = httpx.Client(base_url=base_url, headers=headers)
        
        self.checkout = SantimPayCheckout(self.http_client, merchant_id, private_key)
        self.directPay = SantimPayDirectPay(self.http_client, merchant_id, private_key)
    



from ..types import *
from ..config import Config
from ..exceptions.santimpay_exceptions import *
import jwt as JWT
from time import time
class SantimPayCheckout:

    def __init__(self, http_client, merchant_id, private_key):
    
        self.http_client = http_client
        self.merchant_id = merchant_id
        self.private_key = private_key
    

    # : SantimPayCheckoutRequest SantimPayOptions -> SantimPayCheckoutResponse
    def create(self, santimPayCheckoutRequest: SantimPayCheckoutRequest ,  option: SantimPayOptions = None) -> SantimPayCheckoutResponse: 
        """Create checkout

        Args:
            santimPayCheckoutRequest (SantimPayCheckoutRequest): type of santim pay checkout
            option (SantimPayOptions, optional): checkout options. Defaults to None.

        Raises:
            SantimPayNetworkException: _description_

        Returns:
            SantimPayCheckoutResponse: _description_
        """
        if option == None:
            option = SantimPayOptions(False)
        

        try:

            body = santimPayCheckoutRequest.jsonSerialize()
            body['merchantId'] =  self.merchant_id
            body['signedToken'] = self.generateSignedToken(santimPayCheckoutRequest.amount, self.merchant_id)
            response = self.http_client.post(Config.API_VERSION + "/initiate-payment", json=body)
            url = response.text.replace('\u0026', '&')


            return SantimPayCheckoutResponse.fromJson(url)
        except ConnectionErrorException as e:
            raise  SantimPayNetworkException() #Exception("santim pay n exception")#SantimPayNetworkException()
        except Exception as e:#(ClientException e) {
            #SantimPaySupport.__handleException(e)

            print("exception")

    def generateSignedToken(self, amount, paymentReason):
    
        time_ = time()
        data = {
            'amount' : amount,
            'paymentReason': paymentReason,
            'merchantId': self.merchant_id,
            'generated': time_

        }
        

        jwt = JWT.encode(data, self.privateKey, algorithm='ES256')

        return jwt
    


class SantimPayTransaction:


    def __init__(self, id, transaction_id, payment_type, transaction_status, created_at, updated_at):
    
        self.id = id
        self.transaction_id = transaction_id
        self.payment_type = payment_type
        self.transaction_status = transaction_status
        self.created_at = created_at
        self.updated_at = updated_at
    

    def jsonSerialize(self):
    
        return {
            "id" : self.id,
            "transactionId" : self.transaction_id,
            "paymentType" : self.payment_type,
            "transactionStatus" : self.transaction_status,
            "createdAt" : self.created_at,
            "updatedAt" : self.updated_at,
        }
    
    @staticmethod
    def fromJson(data):
    
        return SantimPayTransaction(data["id"], data["transactionId"], data["paymentType"], data["transactionStatus"], data["createdAt"], data["updatedAt"])
    




from httpx import RequestError as ConnectionErrorException
class SantimPayBadRequestException(Exception):
    def __init__(self, message, previous=None):
        self.msg = message
        # Make sure everything is assigned properly
        super().__init__(message, 0, previous)

    # Custom string representation of object
    def __str__(self):
        return f"{self.__class__.__name__}: {self.msg}\n"
class SantimPayException(Exception):
    def __init__(self, message, previous=None):
        self.msg = message
        # Make sure everything is assigned properly
        super().__init__(message, 0, previous)

    # Custom string representation of object
    def __str__(self):
        return f"{self.__class__.__name__}: {self.msg}\n"
class SantimPayNetworkException(ConnectionErrorException):
    def __init__(self, previous=None):
        # Make sure everything is assigned properly
        super().__init__("NetworkException", 0, previous)

    # Custom string representation of object
    def __str__(self):
        return f"{self.__class__.__name__}: NetworkException\n"

class SantimPayNotFoundException(Exception):
    def __init__(self, message, previous=None):
        self.msg = message
        # Make sure everything is assigned properly
        super().__init__(message, 0, previous)

    # Custom string representation of object
    def __str__(self):
        return f"{self.__class__.__name__}: {self.msg}\n"

class SantimPayUnAuthorizedException(Exception):
    def __init__(self, message, previous=None):
        self.msg = message
        # Make sure everything is assigned properly
        super().__init__(message, 0, previous)

    # Custom string representation of object
    def __str__(self):
        return f"{self.__class__.__name__}: {self.msg}\n"


from setuptools import setup, find_packages

VERSION = '1.0.1'
DESCRIPTION = 'SantimPay Unofficial package'
LONG_DESCRIPTION = ''
with open('README.md', 'r') as r:
    LONG_DESCRIPTION = r.read()

setup(
    name='santimPay',
    version=VERSION,
    author='Nahom Dereje',
    license='MIT',
    author_email='dev@nahom.eu.org',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    url="https://github.com/nahom-d54/SantimPay.git",
    install_requires=[
        "PyJWT==2.8.0",
        "httpx==0.27.0"
    ],
    keywords=['SantimPay', "Python", "PaymentGateway", "EthiopiaPaymentGateway"],
    classifiers= [
                "Development Status :: 3 - Alpha",
                "Intended Audience :: Education",
                "Programming Language :: Python :: 3",
                "Operating System :: MacOS :: MacOS X",
                "Operating System :: Microsoft :: Windows"
            ]
)



import urllib.parse
from . import SantimPayTransaction

urldecode = urllib.parse.unquote
class SantimPayTransferResponse:

    def __init__(
        self,
        session_id,
        url,
        otp,
        transaction
    ):
        self.transaction = transaction
        self.session_id = session_id
        self.url = url
        self.otp = otp
    

    def jsonSerialize(self):
    
        return {
            "session_id" : self.session_id,
            "url" : self.url,
            "otp" : self.otp,
            "transaction" : self.transaction,
        }
    
    @staticmethod
    def fromJson(data):
    
        return SantimPayTransferResponse(data["sessionId"], urldecode(data["url"]) if data.get("url") else "", urldecode(data["otp"]) if data.get("otp") else "", SantimPayTransaction.fromJson(data["transaction"]) if data.get("transaction") else None)
    



from .direct_pay.santimpay_direct import SantimPayDirect

class SantimPayDirectPay:

    def __init__(self, http_client, merchant_id, private_key):
    
        self.http_client = http_client
        self.telebirr = SantimPayDirect(self.http_client, merchant_id, private_key, 'Telebirr')
        self.cbe = SantimPayDirect(self.http_client, merchant_id, private_key, 'CBE Birr')
    